@@ -840,8 +840,8 @@ enumerate_mimetypes_dir (const char *dir,
  * g_content_types_get_registered:
  * 
  * Gets a list of strings containing all the registered content types
- * known to the system.
- * 
+ * known to the system. The list and its data should be freed using 
+ * @g_list_foreach(list, g_free, NULL) and @g_list_free(list)
  * Returns: #GList of the registered content types.
  **/  
 GList *
@@ -854,7 +854,7 @@ g_content_types_get_registered (void)
   int i;
   GList *l;
 
-  mimetypes = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+  mimetypes = g_hash_table_new (g_str_hash, g_str_equal);
 
   enumerate_mimetypes_dir (g_get_user_data_dir (), mimetypes);
   dirs = g_get_system_data_dirs ();
