@@ -24,11 +24,30 @@
  * SECTION:gfileinfo
  * @short_description: File Information and Attributes
  * @include: gio.h
- * @see_also: #GFile
+ * @see_also: #GFile, <link linkend="gio-GFileAttribute">GFileAttribute</link>
  * 
  * Functionality for manipulating basic metadata for files. #GFileInfo
  * implements methods for getting information that all files should 
  * contain, and allows for manipulation of extended attributes. 
+ *
+ * See <link linkend="gio-GFileAttribute">GFileAttribute</link> for more
+ * information on how GIO handles file attributes.
+ *
+ * To obtain a #GFileInfo for a #GFile, use g_file_query_info() (or its 
+ * async variant). To obtain a #GFileInfo for a file input or output 
+ * stream, use g_file_input_stream_query_info() or 
+ * g_file_output_stream_query_info() (or their async variants).
+ *
+ * To change the actual attributes of a file, you should then set the 
+ * attribute in the #GFileInfo and call g_file_set_attributes_from_info() 
+ * or g_file_set_attributes_async() on a GFile.
+ *
+ * However, not all attributes can be changed in the file. For instance, 
+ * the actual size of a file cannot be changed via g_file_info_set_size(). 
+ * You may call g_file_query_settable_attributes() and 
+ * g_file_query_writable_namespaces() to discover the settable attributes 
+ * of a particular file at runtime.
+ *
  * #GFileAttributeMatcher allows for searching through a #GFileInfo for 
  * attributes.
  **/
@@ -447,8 +466,8 @@ g_file_info_find_value_by_name (GFileInfo  *info,
  * 
  * Checks if a file info structure has an attribute named @attribute.
  * 
- * Returns: %TRUE if @GFileInfo has an attribute named @attribute, 
- * %FALSE otherwise.
+ * Returns: %TRUE if @Ginfo has an attribute named @attribute, 
+ *     %FALSE otherwise.
  **/
 gboolean
 g_file_info_has_attribute (GFileInfo  *info,
@@ -533,7 +552,6 @@ g_file_info_get_attribute_type (GFileInfo  *info,
  * @attribute: a file attribute key.
  * 
  * Removes all cases of @attribute from @info if it exists.
- *
  **/
 void
 g_file_info_remove_attribute (GFileInfo  *info,
@@ -558,11 +576,24 @@ g_file_info_remove_attribute (GFileInfo  *info,
     }
 }
 
+/**
+ * g_file_info_get_attribute_data:
+ * @info: a #GFileInfo
+ * @attribute: a file attribute key
+ * @type: return location for the attribute type, or %NULL
+ * @value_pp: return location for the attribute value, or %NULL
+ * @status: return location for the attribute status, or %NULL
+ *
+ * Gets the attribute type, value and status for an attribute key.
+ *
+ * Returns: %TRUE if @info has an attribute named @attribute, 
+ *      %FALSE otherwise.
+ */
 gboolean
-g_file_info_get_attribute_data (GFileInfo  *info,
-				const char *attribute,
-				GFileAttributeType *type,
-				gpointer   *value_pp,
+g_file_info_get_attribute_data (GFileInfo            *info,
+				const char           *attribute,
+				GFileAttributeType   *type,
+				gpointer             *value_pp,
 				GFileAttributeStatus *status)
 {
   GFileAttributeValue *value;
@@ -583,6 +614,17 @@ g_file_info_get_attribute_data (GFileInfo  *info,
   return TRUE;
 }
 
+/** 
+ * g_file_info_get_attribute_status:
+ * @info: a #GFileInfo
+ * @attribute: a file attribute key
+ *
+ * Gets the attribute status for an attribute key.
+ *
+ * Returns: a #GFileAttributeStatus for the given @attribute, or 
+ *    %G_FILE_ATTRIBUTE_STATUS_UNSET if the key is invalid.
+ *
+ */
 GFileAttributeStatus
 g_file_info_get_attribute_status (GFileInfo  *info,
 				  const char *attribute)
@@ -596,7 +638,7 @@ g_file_info_get_attribute_status (GFileInfo  *info,
   if (val)
     return val->status;
 
-  return 0;
+  return G_FILE_ATTRIBUTE_STATUS_UNSET;
 }
 
 
@@ -620,8 +662,8 @@ _g_file_info_get_attribute_value (GFileInfo  *info,
  * This escapes things as needed to make the string valid
  * utf8.
  * 
- * Returns: a utf8 string associated with the given @attribute.
- *    When you're done with the string it must be freed.
+ * Returns: a UTF-8 string associated with the given @attribute.
+ *    When you're done with the string it must be freed with g_free().
  **/
 char *
 g_file_info_get_attribute_as_string (GFileInfo  *info,
@@ -1894,11 +1936,11 @@ g_file_attribute_matcher_new (const char *attributes)
 GFileAttributeMatcher *
 g_file_attribute_matcher_ref (GFileAttributeMatcher *matcher)
 {
-  g_return_val_if_fail (matcher != NULL, NULL);
-  g_return_val_if_fail (matcher->ref > 0, NULL);
-
-  g_atomic_int_inc (&matcher->ref);
-  
+  if (matcher)
+    {
+      g_return_val_if_fail (matcher->ref > 0, NULL);
+      g_atomic_int_inc (&matcher->ref);
+    }
   return matcher;
 }
 
@@ -1913,15 +1955,17 @@ g_file_attribute_matcher_ref (GFileAttributeMatcher *matcher)
 void
 g_file_attribute_matcher_unref (GFileAttributeMatcher *matcher)
 {
-  g_return_if_fail (matcher != NULL);
-  g_return_if_fail (matcher->ref > 0);
-
-  if (g_atomic_int_dec_and_test (&matcher->ref))
+  if (matcher)
     {
-      if (matcher->more_sub_matchers)
-	g_array_free (matcher->more_sub_matchers, TRUE);
+      g_return_if_fail (matcher->ref > 0);
       
-      g_free (matcher);
+      if (g_atomic_int_dec_and_test (&matcher->ref))
+	{
+	  if (matcher->more_sub_matchers)
+	    g_array_free (matcher->more_sub_matchers, TRUE);
+	  
+	  g_free (matcher);
+	}
     }
 }
 
@@ -1941,10 +1985,10 @@ g_file_attribute_matcher_matches_only (GFileAttributeMatcher *matcher,
 {
   guint32 id;
 
-  g_return_val_if_fail (matcher != NULL, FALSE);
   g_return_val_if_fail (attribute != NULL && *attribute != '\0', FALSE);
 
-  if (matcher->all)
+  if (matcher == NULL ||
+      matcher->all)
     return FALSE;
   
   id = lookup_attribute (attribute);
@@ -2014,9 +2058,12 @@ gboolean
 g_file_attribute_matcher_matches (GFileAttributeMatcher *matcher,
 				  const char            *attribute)
 {
-  g_return_val_if_fail (matcher != NULL, FALSE);
   g_return_val_if_fail (attribute != NULL && *attribute != '\0', FALSE);
 
+  /* We return a NULL matcher for an empty match string, so handle this */
+  if (matcher == NULL)
+    return FALSE;
+  
   if (matcher->all)
     return TRUE;
   
@@ -2047,8 +2094,11 @@ g_file_attribute_matcher_enumerate_namespace (GFileAttributeMatcher *matcher,
   int ns_id;
   int i;
   
-  g_return_val_if_fail (matcher != NULL, FALSE);
   g_return_val_if_fail (ns != NULL && *ns != '\0', FALSE);
+
+  /* We return a NULL matcher for an empty match string, so handle this */
+  if (matcher == NULL)
+    return FALSE;
   
   if (matcher->all)
     return TRUE;
@@ -2091,8 +2141,10 @@ g_file_attribute_matcher_enumerate_next (GFileAttributeMatcher *matcher)
 {
   int i;
   SubMatcher *sub_matcher;
-  
-  g_return_val_if_fail (matcher != NULL, NULL);
+
+  /* We return a NULL matcher for an empty match string, so handle this */
+  if (matcher == NULL)
+    return NULL;
 
   while (1)
     {
