@@ -205,8 +205,9 @@ get_default_local_file_monitor (gpointer data)
  * Returns: a new #GFileMonitor for the given @pathname. 
  **/
 GFileMonitor*
-_g_local_file_monitor_new (const char        *pathname,
-			   GFileMonitorFlags  flags)
+_g_local_file_monitor_new (const char         *pathname,
+			   GFileMonitorFlags   flags,
+			   GError            **error)
 {
   static GOnce once_init = G_ONCE_INIT;
   GTypeClass *type_class;
@@ -220,6 +221,8 @@ _g_local_file_monitor_new (const char        *pathname,
   monitor = NULL;
   if (type != G_TYPE_INVALID)
     monitor = G_FILE_MONITOR (g_object_new (type, "filename", pathname, NULL));
+  else
+    g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, _("Unable to find default local file monitor type"));
 
   /* This is non-null on first pass here. Unref the class now.
    * This is to avoid unloading the module and then loading it
