@@ -273,6 +273,7 @@ guess_system_internal (const char *mountpoint,
     "tmpfs",
     "usbfs",
     "nfsd",
+    "rpc_pipefs",
     NULL
   };
   const char *ignore_devices[] = {
@@ -1190,7 +1191,7 @@ g_unix_mount_monitor_init (GUnixMountMonitor *monitor)
   if (get_fstab_file () != NULL)
     {
       file = g_file_new_for_path (get_fstab_file ());
-      monitor->fstab_monitor = g_file_monitor_file (file, 0, NULL);
+      monitor->fstab_monitor = g_file_monitor_file (file, 0, NULL, NULL);
       g_object_unref (file);
       
       g_signal_connect (monitor->fstab_monitor, "changed", (GCallback)fstab_file_changed, monitor);
@@ -1199,7 +1200,7 @@ g_unix_mount_monitor_init (GUnixMountMonitor *monitor)
   if (get_mtab_monitor_file () != NULL)
     {
       file = g_file_new_for_path (get_mtab_monitor_file ());
-      monitor->mtab_monitor = g_file_monitor_file (file, 0, NULL);
+      monitor->mtab_monitor = g_file_monitor_file (file, 0, NULL, NULL);
       g_object_unref (file);
       
       g_signal_connect (monitor->mtab_monitor, "changed", (GCallback)mtab_file_changed, monitor);
@@ -1769,7 +1770,7 @@ g_unix_mount_guess_name (GUnixMountEntry *mount_entry)
 GIcon *
 g_unix_mount_guess_icon (GUnixMountEntry *mount_entry)
 {
-  return g_themed_icon_new (type_to_icon (g_unix_mount_guess_type (mount_entry), FALSE));
+  return g_themed_icon_new_with_default_fallbacks (type_to_icon (g_unix_mount_guess_type (mount_entry), FALSE));
 }
 
 /**
@@ -1806,7 +1807,7 @@ g_unix_mount_point_guess_name (GUnixMountPoint *mount_point)
 GIcon *
 g_unix_mount_point_guess_icon (GUnixMountPoint *mount_point)
 {
-  return g_themed_icon_new (type_to_icon (g_unix_mount_point_guess_type (mount_point), TRUE));
+  return g_themed_icon_new_with_default_fallbacks (type_to_icon (g_unix_mount_point_guess_type (mount_point), TRUE));
 }
 
 /**
@@ -1830,6 +1831,40 @@ g_unix_mount_guess_can_eject (GUnixMountEntry *mount_entry)
   return FALSE;
 }
 
+/**
+ * g_unix_mount_guess_should_display:
+ * @mount_entry: a #GUnixMountEntry
+ * 
+ * Guesses whether a Unix mount should be displayed in the UI.
+ *
+ * Returns: %TRUE if @mount_entry is deemed to be displayable.
+ */
+gboolean
+g_unix_mount_guess_should_display (GUnixMountEntry *mount_entry)
+{
+  GUnixMountType guessed_type;
+
+  /* Never display internal mountpoints */
+  if (g_unix_mount_is_system_internal (mount_entry))
+    return FALSE;
+
+  /* Only display things that look "removable" or
+     things in /media (which are generally user mountable) */
+  guessed_type = g_unix_mount_guess_type (mount_entry);
+  if (guessed_type == G_UNIX_MOUNT_TYPE_IPOD ||
+      guessed_type == G_UNIX_MOUNT_TYPE_CDROM ||
+      guessed_type == G_UNIX_MOUNT_TYPE_FLOPPY ||
+      guessed_type == G_UNIX_MOUNT_TYPE_ZIP ||
+      guessed_type == G_UNIX_MOUNT_TYPE_JAZ ||
+      guessed_type == G_UNIX_MOUNT_TYPE_CAMERA ||
+      guessed_type == G_UNIX_MOUNT_TYPE_MEMSTICK ||
+      (mount_entry->mount_path != NULL &&
+       g_str_has_prefix (mount_entry->mount_path, "/media")))
+    return TRUE;
+
+  return FALSE;
+}
+
 /**
  * g_unix_mount_point_guess_can_eject:
  * @mount_point: a #GUnixMountPoint
